! Utility module to specify the GoldSim parameter constants
MODULE gs_parameters
    IMPLICIT NONE 
    ! Parameters to identify the method types, which indicate the phase of the
    ! simulation that is currently in progress.
    ! 
    ! INITIALIZE - Called after DLL is loaded and before each realization.
    ! CALCULATE - Called during the simulation, each time the inputs change.
    ! REPORT_VERSION - Called after DLL load to report the external fcn version
    !   number.
    ! REPORT_ARGUMENTS - Called after DLL load to report the number of input 
    !   arguments.
    ! CLEANUP - Called before the DLL is unloaded.
    INTEGER, PARAMETER :: INITIALIZE = 0
    INTEGER, PARAMETER :: CALCULATE = 1
    INTEGER, PARAMETER :: REPORT_VERSION = 2
    INTEGER, PARAMETER :: REPORT_ARGUMENTS = 3
    INTEGER, PARAMETER :: CLEANUP = 99
    ! Parameters to identify the return codes for external functions.
    !
    ! SUCCESS – Call completed successfully.
    ! CLEANUP_NOW - Call was successful, but GoldSim should clean up and unload
    !   the DLL immediately.
    ! FAILURE - Failure (no error information returned).
    ! FAILURE_WITH_MSG – Failure, with DLL-supplied error message available.
    ! Address of error message is returned in the first element of the output 
    !   arguments array.
    ! INCREASE_MEMORY - Failed because the memory allocated for output 
    !   arguments is too small. GoldSim will increase the size of the output 
    !   argument array and try again.
    INTEGER, PARAMETER :: SUCCESS = 0
    INTEGER, PARAMETER :: FAILURE = 1
    INTEGER, PARAMETER :: CLEANUP_NOW = 99
    INTEGER, PARAMETER :: FAILURE_WITH_MSG = -1
    INTEGER, PARAMETER :: INCREASE_MEMORY = -2
END MODULE gs_parameters

! subroutine to report the version
SUBROUTINE ReportVersion( TheVersion, outargs ) bind (C, name="ReportVersion")
    USE, INTRINSIC :: iso_c_binding 
    IMPLICIT NONE 
    ! arguments
    REAL(C_DOUBLE), INTENT(IN) :: TheVersion 
    REAL(C_DOUBLE), DIMENSION(1), INTENT(OUT) :: outargs 
    ! start of function logic
    outargs = [ TheVersion ];
    RETURN
END SUBROUTINE ReportVersion 

SUBROUTINE RepSumArgs( outargs ) bind(C, name="RepSumArgs")
    USE, INTRINSIC :: iso_c_binding 
    IMPLICIT NONE 
    ! parameters
    REAL(C_DOUBLE), PARAMETER :: NumIn = 2.0;
    REAL(C_DOUBLE), PARAMETER :: NumOut = 1.0;
    ! arguments
    REAL(C_DOUBLE), DIMENSION(2), INTENT(OUT) :: outargs 
    ! function
    outargs = [ NumIn, NumOut ]
    RETURN
END SUBROUTINE RepSumArgs

SUBROUTINE Sum ( status, inargs, outargs ) bind (C, name="Sum")
    USE, INTRINSIC :: iso_c_binding 
    USE gs_parameters
    IMPLICIT NONE 
    ! parameters
    INTEGER, PARAMETER :: double = 8   ! compiler dependent size identifier
    INTEGER(KIND=double), PARAMETER :: NINPUTS = 2 ! Two scalar inputs expected
    INTEGER(KIND=double), PARAMETER :: NOUTPUTS = 1 ! One scalar outputs returned
    ! variables
    REAL(C_DOUBLE) :: x, y, Calc1 
    INTEGER(C_INT) :: RetStatus
    ! arguments
    INTEGER(C_INT), INTENT(OUT):: status 
    REAL(C_DOUBLE), DIMENSION(NOUTPUTS), INTENT(OUT) :: outargs 
    REAL(C_DOUBLE), DIMENSION(NINPUTS), INTENT(IN) :: inargs 
    ! start of function logic
    x = inargs(1) 
    y = inargs(2) 
    Calc1 = x + y 
    outargs = [ Calc1 ]
    RetStatus = SUCCESS
    status = RetStatus
    RETURN
END SUBROUTINE Sum 

SUBROUTINE RepProdSumArgs( outargs ) bind(C, name="RepProdSumArgs")
    USE, INTRINSIC :: iso_c_binding 
    IMPLICIT NONE 
    ! parameters
    REAL(C_DOUBLE), PARAMETER :: NumIn = 2.0;
    REAL(C_DOUBLE), PARAMETER :: NumOut = 2.0;
    ! arguments
    REAL(C_DOUBLE), DIMENSION(2), INTENT(OUT) :: outargs 
    ! function
    outargs = [ NumIn, NumOut ]
    RETURN
END SUBROUTINE RepProdSumArgs

SUBROUTINE ProdSum ( status, inargs, outargs ) bind(C, name="ProdSum")
    USE, INTRINSIC :: iso_c_binding 
    USE gs_parameters 
    IMPLICIT NONE 
    ! parameters 
    INTEGER, PARAMETER :: double = 8   ! compiler dependent size identifier
    INTEGER(KIND=double), PARAMETER :: NINPUTS = 2 ! Two scalar inputs expected
    INTEGER(KIND=double), PARAMETER :: NOUTPUTS = 2 ! Two scalar outputs returned
    ! variables
    REAL(C_DOUBLE) :: x, y, Calc1, Calc2  
    INTEGER(C_INT) :: RetStatus
    ! arguments
    INTEGER(C_INT), INTENT(OUT):: status 
    REAL(C_DOUBLE), DIMENSION(NOUTPUTS), INTENT(OUT) :: outargs 
    REAL(C_DOUBLE), DIMENSION(NINPUTS), INTENT(IN) :: inargs
    ! start of function logic
    x = inargs(1) 
    y = inargs(2) 
    Calc1 = x + y 
    Calc2 = x * y
    outargs = [ Calc1, Calc2 ]
    RetStatus = SUCCESS
    status = RetStatus
    RETURN
END SUBROUTINE ProdSum

SUBROUTINE RepHalfArgs( outargs ) bind(C, name="RepHalfArgs")
    USE, INTRINSIC :: iso_c_binding 
    IMPLICIT NONE 
    ! parameters
    REAL(C_DOUBLE), PARAMETER :: NumIn = 1.0;
    REAL(C_DOUBLE), PARAMETER :: NumOut = 1.0;
    ! arguments
    REAL(C_DOUBLE), DIMENSION(2), INTENT(OUT) :: outargs 
    ! function
    outargs = [ NumIn, NumOut ]
    RETURN
END SUBROUTINE RepHalfArgs

SUBROUTINE Half( status, inargs, outargs ) bind(C, name="Half")
    USE, INTRINSIC :: iso_c_binding 
    USE gs_parameters 
    IMPLICIT NONE 
    ! parameters 
    INTEGER, PARAMETER :: double = 8   ! compiler dependent size identifier
    INTEGER(KIND=double), PARAMETER :: NINPUTS = 1 ! Two scalar inputs expected
    INTEGER(KIND=double), PARAMETER :: NOUTPUTS = 1 ! Two scalar outputs returned
    ! declare the function
    REAL(KIND=double) :: FuncHalf 
    ! variables
    REAL(C_DOUBLE) :: x, Calc1 
    INTEGER(C_INT) :: RetStatus
    ! arguments
    INTEGER(C_INT), INTENT(OUT):: status 
    REAL(C_DOUBLE), DIMENSION(NOUTPUTS), INTENT(OUT) :: outargs 
    REAL(C_DOUBLE), DIMENSION(NINPUTS), INTENT(IN) :: inargs
    ! start of function logic
    x = inargs(1)
    Calc1 = FuncHalf(x)
    outargs = [ Calc1 ]
    RetStatus = SUCCESS
    status = RetStatus
    RETURN 
END SUBROUTINE Half 

REAL(KIND=8) FUNCTION FuncHalf(x) 
    IMPLICIT NONE 
    ! parameters 
    INTEGER, PARAMETER :: double = 8   ! compiler dependent size identifier
    REAL(KIND=double), PARAMETER :: ByHalf = 2.0 
    ! arguments
    REAL(KIND=double), INTENT(IN) :: x 
    ! function body 
    FuncHalf = x / ByHalf 
    RETURN 
END FUNCTION FuncHalf 
!EOF 