// Global enumerations, useful for C-style implementations
//
// XFMethodID identifies the method types, used to identify the phase of the
//  simulation that is currently in progress.
//
// XF_INITIALIZE - Called after DLL is loaded and before each realization.
// XF_CALCULATE - Called during the simulation, each time the inputs change.
// XF_REP_VERSION - Called after DLL load to report the external fcn version 
//      number.
// XF_REP_ARGUMENTS - Called after DLL load to report the number of input and 
//      output arguments.
// XF_CLEANUP - Called before the DLL is unloaded.
//
enum XFMethodID
{
    XF_INITIALIZE = 0,
    XF_CALCULATE = 1, 
    XF_REP_VERSION = 2, 
    XF_REP_ARGUMENTS = 3,
    XF_CLEANUP = 99
};

// XFStatusID identifies the return codes for external functions.
//
// XF_SUCCESS – Call completed successfully.
// XF_CLEANUP_NOW - Call was successful, but GoldSim should clean up and unload
//      the DLL immediately.
// XF_FAILURE - Failure (no error information returned).
// XF_FAILURE_WITH_MSG – Failure, with DLL-supplied error message available.
//      Address of error message is returned in the first element of the output
//      arguments array.
// XF_INCREASE_MEMORY - Failed because the memory allocated for output arguments
//      is too small. GoldSim will increase the size of the output argument 
//      array and try again.
//
enum XFStatusID
{
    XF_SUCCESS = 0, 
    XF_FAILURE = 1, 
    XF_CLEANUP_NOW = 99, 
    XF_FAILURE_WITH_MSG = -1,
    XF_INCREASE_MEMORY = -2
};

// declare version number parameter
double VersionNumber = 1.0;
// declare the fortran subroutines
void ReportVersion( double* TheVersion, double* outargs );
void RepSumArgs( double* outargs );
void Sum ( int* status, double* inargs, double* outargs );
void RepProdSumArgs( double* outargs );
void ProdSum( int* status, double* inargs, double* outargs );
void RepHalfArgs( double* outargs );
void Half( int* status, double* inargs, double* outargs );

///////////////////////////////////////////////////////////////////////////////
// CustomFortranDLLs
// C Language implementation DLL wrapper for Fortran subroutines. Assume that
//      will be compiling with gfortran and gcc. Have subroutines for each
//      testing function/implementation.
//-----------------------------------------------------------------------------
extern void __declspec(dllexport) CustomFortranDLLSum( int methodID, 
                                int* status, double* inargs, double* outargs )
{
    *status = XF_SUCCESS;
    switch ( methodID )
    {
        case XF_INITIALIZE:
            break; // nothing required
        case XF_REP_VERSION:
            ReportVersion( &VersionNumber, outargs );
            break;
        case XF_REP_ARGUMENTS:
            RepSumArgs( outargs );
            break;
        case XF_CALCULATE:
            Sum( status, inargs, outargs );
            break;
        case XF_CLEANUP:
            break; // No clean-up required
        default:
            *status = XF_FAILURE;
            break;
    }
}

extern void __declspec(dllexport) CustomFortranDLLProdSum( int methodID, 
                                int* status, double* inargs, double* outargs )
{
    *status = XF_SUCCESS;
    switch ( methodID )
    {
        case XF_INITIALIZE:
            break; // nothing required
        case XF_REP_VERSION:
            ReportVersion( &VersionNumber, outargs );
            break;
        case XF_REP_ARGUMENTS:
            RepProdSumArgs( outargs );
            break;
        case XF_CALCULATE:
            ProdSum( status, inargs, outargs );
            break;
        case XF_CLEANUP:
            break; // No clean-up required
        default:
            *status = XF_FAILURE;
            break;
    }
}

extern void __declspec(dllexport) CustomFortranDLLHalf( int methodID, 
                                int* status, double* inargs, double* outargs )
{
    *status = XF_SUCCESS;
    switch ( methodID )
    {
        case XF_INITIALIZE:
            break; // nothing required
        case XF_REP_VERSION:
            ReportVersion( &VersionNumber, outargs );
            break;
        case XF_REP_ARGUMENTS:
            RepHalfArgs( outargs );
            break;
        case XF_CALCULATE:
            Half( status, inargs, outargs );
            break;
        case XF_CLEANUP:
            break; // No clean-up required
        default:
            *status = XF_FAILURE;
            break;
    }
}
// EOF